#include <GL/glut.h>
#include "GameObj.h"
#include "GameObjTimed.h"
#include "Bullet.h"
#include "Ast.h"
#include "Exp.h"
#include "Fragment.h"
#include "Ast3DFract.h"


#ifdef _WIN32
#define drand48() (((float) rand())/((float) RAND_MAX))
#define srand48(x) (srand((x)))
#endif

int Ast3DFract::Draw()
{
	glPushMatrix();
	glTranslate(p);
	glRotatef(angle,axis.x(),axis.y(),axis.z());
	glScalef(size,size,size);
	glColor3f(.8,.8,.7);
	int nt=M.DrawDL();
	glPopMatrix();
	return nt;
}
/*
 v0


mp01     mp02
 

 v1     mp12       v2

*/

void Ast3DFract::Split(int seed,float strenght)
{
	Point3f BC(0,0,0); // The BaryCenter of the Asteroid
	vector<Triangle>::iterator i;
	for(i=M.T.begin();i!=M.T.end();i++)	{
			BC+=(*i).v[0].v;
			BC+=(*i).v[1].v;
			BC+=(*i).v[2].v;
	}
	BC/=(M.T.size()*3);
	Mesh M2;
	Point3f mp01,mp02,mp12;

	for(i=M.T.begin();i!=M.T.end();i++)	{
		mp01=MidPoint((*i).v[0].v,(*i).v[1].v,BC,seed,strenght);
		mp02=MidPoint((*i).v[0].v,(*i).v[2].v,BC,seed,strenght);
		mp12=MidPoint((*i).v[1].v,(*i).v[2].v,BC,seed,strenght);
		M2.T.push_back(Triangle((*i).v[0].v,mp01,mp02));
		M2.T.push_back(Triangle((*i).v[1].v,mp12,mp01));
		M2.T.push_back(Triangle((*i).v[2].v,mp02,mp12));
		M2.T.push_back(Triangle(mp12,mp02,mp01));
	}
	M=M2;
}

Point3f Ast3DFract::MidPoint(Point3f &p0, Point3f &p1, Point3f const &BC,int seed,float strenght)
{
	bool swapped=false;
	if(p0<p1){
		swap(p0,p1);
		swapped=true;
	}
	Point3f mp=(p0+p1)/2.0;
	float r=(Distance(p0,BC)+Distance(p1,BC))/2.0; // Radius
  float l=Distance(p0,p1);	// Edge Lenght
  l=l*strenght;
	srand48(seed + p0.Hash() + p1.Hash()); 
	float d=drand48()-0.5f;		// Displacement depends only on the two vertexes p0,p1
	
	mp=BC+Normalize(mp-BC)*(r+d*l/2.0);
	//mp=BC+Normalize(mp-BC)*r;
	if(swapped) swap(p0,p1);
	return mp;
}
 

void Ast3DFract::Explode(list<Ast *> &AL, list<GameObj *> &OL)
{
	const int FragNum=5;
	active=false;
	Exp *e =new Exp();
	e->p=p;
	e->LightOn();
	OL.push_back(e);	
	if(size>=Ast::MediumSize)
			{
				assert(Sub[0]!=0 && Sub[1]!=0);
				Ast *a=Sub[0];
				Point3f nv;
				nv.x()=drand48()*40 - 20;	
				nv.y()=drand48()*40 - 20;
				nv.z()=0;
				a->av=20+drand48()*70;
				a->axis=Normalize(Point3f(drand48(),drand48(),drand48()));
				a->p=p;
				a->v=v+nv;
				AL.push_front(a);
				a=Sub[1];
				a->av=20+drand48()*70;
				a->axis=Normalize(Point3f(drand48(),drand48(),drand48()));
				a->p=p;
				a->v=v-nv;
				AL.push_front(a);
				Fragment *f;
				for(int i=0; i<FragNum;i++)
				{
					f=new Fragment();
					f->p=p;
					f->v=Point3f(drand48()-.5f,drand48()-.5f,0)*30;
					f->axis=Normalize(Point3f(drand48(),drand48(),drand48()));
					f->av=40+drand48()*100;
					OL.push_front(f);
				}
			}
}


void Ast3DFract::GenerateSub()
{
	if(size<=SmallSize) return;
	Sub.resize(2);
	if(size>=LargeSize){
		Sub[0]= new Ast3DFract("data/tetra.raw",Ast::MediumSize);
		Sub[1]= new Ast3DFract("data/tetra.raw",Ast::MediumSize);
		Sub[0]->GenerateSub();
		Sub[1]->GenerateSub();
	}
	else {
		Sub[0]= new Ast3DFract("data/tetra.raw",Ast::SmallSize);
		Sub[1]= new Ast3DFract("data/tetra.raw",Ast::SmallSize);
	}
}